(* 
 * Dusty deBoer, Kansas State University.
 *
 * Based on basicwin.sml, (C) 1990 J.H. Reppy; and goodbye.sml, (C) 1990 AT&T.
 *)

structure XDEMO : sig

    val doit  : string option * string list -> OS.Process.status
    val main  : (string * string list) -> OS.Process.status

  end = struct

    structure EXB = EXeneBase

fun init (dpyOpt,args) =
    let
    val root  = Widget.mkRoot(GetDpy.getDpy(dpyOpt))
        handle EXB.BadAddr s =>
            (TextIO.print s; RunCML.shutdown OS.Process.failure)
               
    val styleView = Styles.mkView{name = Styles.styleName["demofocus"],aliases = nil}
    val view      = (styleView, (Widget.styleOf root))
    
    fun quit ()  = (Widget.delRoot root; RunCML.shutdown OS.Process.success)
    
    val quitBttn1 = Button.textBtn (root, view, 
                    [([], Attrs.attr_label, Attrs.AV_Str "Handle Quit")])
    val quitEvt1  = Button.evtOf quitBttn1
    
    val smplEdit1 = SimpleEdit.simpleEdit (root, view, []) "111"
    val smplEdit2 = SimpleEdit.simpleEdit (root, view, []) "222"
    val smplEdit3 = SimpleEdit.simpleEdit (root, view, []) "333"
    val smplEdit4 = SimpleEdit.simpleEdit (root, view, []) "444"
    val smplEdit5 = SimpleEdit.simpleEdit (root, view, []) "555"
    
    val ff1 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit1),(SimpleEdit.focusableOf smplEdit1))
    val ff2 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit2),(SimpleEdit.focusableOf smplEdit2))
    val ff3 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit3),(SimpleEdit.focusableOf smplEdit3))
    val ff4 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit4),(SimpleEdit.focusableOf smplEdit4))
    val ff5 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit5),(SimpleEdit.focusableOf smplEdit5))
        
    val layout =
        Box.layout (root, view, []) (Box.VtCenter[
            Box.WBox(Button.widgetOf quitBttn1),
            Box.WBox(FocusFrame.widgetOf ff1),
            Box.WBox(FocusFrame.widgetOf ff2),
            Box.WBox(FocusFrame.widgetOf ff3),
            Box.WBox(FocusFrame.widgetOf ff4),
            Box.WBox(FocusFrame.widgetOf ff5)
        ])
    val shellArgs =
        [([], Attrs.attr_title, Attrs.AV_Str "eXene Focus Demo"),
         ([], Attrs.attr_iconName, Attrs.AV_Str "demo-focus")]
    val shell = Shell.shell (root, view, shellArgs) (Box.widgetOf layout)
    
    val id1 = Shell.addFocusableFirst shell (FocusFrame.focusableOf ff1)
    val id2 = Shell.addFocusableAfter shell (id1,FocusFrame.focusableOf ff2)
    val id3 = Shell.addFocusableAfter shell (id2,FocusFrame.focusableOf ff3)
    val id4 = Shell.addFocusableAfter shell (id3,FocusFrame.focusableOf ff4)
    val id5 = Shell.addFocusableAfter shell (id4,FocusFrame.focusableOf ff5)
    
    val hints = Shell.mkHints{size_hints=[],wm_hints=[ICCC.HINT_Input(true)]}
    val _     = Shell.setWMHints shell hints
    val cmEvt = Shell.deletionEvent shell 
    
    fun loop():unit =
        let 
        fun handleQuit (Button.BtnUp _)   = (TextIO.print " [demo-focus quitting]\n"; quit())
          | handleQuit (_) = (loop()) 
        in CML.select
           [CML.wrap(quitEvt1, handleQuit),
            CML.wrap(cmEvt, quit)
          ]
        end
    in
        Shell.init shell;
        loop()
    end

  fun doit (dpyOpt,args) = 
      (RunCML.doit (fn () => (init (dpyOpt,args)), NONE))

  fun main (prog, "-display"::(server::args)) = 
            ((TextIO.print ("display="^server)); doit(SOME server,args))
    | main (prog, args) = doit(NONE,args)

end
